--======================================================================================
--Listing of All Clustered Indexes w/ GUIDs and Binary Typed Columns in Clustered Index
--======================================================================================
SELECT object_name(AC.[object_id]) AS [table_name]
	, I.name
	, I.is_primary_key
	, I.fill_factor
	, AC.name AS column_name
	, T.name AS [system_data_type]
	, IC.index_column_id AS index_column_order 
	, 'Clustered Index w/GUID or Binary' AS notes
FROM sys.index_columns IC 
	INNER JOIN sys.indexes I 
		ON IC.[object_id] = I.[object_id] AND IC.index_id = I.index_id
	INNER JOIN sys.all_columns AC ON IC.[object_id] = AC.[object_id] AND IC.column_id = AC.column_id
	INNER JOIN sys.types T ON AC.system_type_id = T.system_type_id
WHERE I.index_id = 1
	AND T.name IN ('uniqueidentifier', 'binary')
ORDER BY 1;



--======================================================================================================
--Listing of All Clustered Indexes w/ PK Constraint w/ GUIDs and Binary Typed Columns in Clustered Index
--======================================================================================================
SELECT object_name(AC.[object_id]) AS [table_name]
	, I.name
	, I.is_primary_key
	, I.fill_factor
	, AC.name AS column_name
	, T.name AS [system_data_type]
	, IC.index_column_id AS index_column_order
	, 'Clustered Index as Primary Key w/GUID or Binary' AS notes 
FROM sys.index_columns IC 
	INNER JOIN sys.indexes I 
		ON IC.[object_id] = I.[object_id] AND IC.index_id = I.index_id
	INNER JOIN sys.all_columns AC ON IC.[object_id] = AC.[object_id] AND IC.column_id = AC.column_id
	INNER JOIN sys.types T ON AC.system_type_id = T.system_type_id
WHERE I.index_id = 1
	AND T.name IN ('uniqueidentifier', 'binary')
	AND I.is_primary_key = 1
ORDER BY 1;



--======================================================================================================
-- Listing of All Columns in Clustered Indexes w/PK Constraint Having GUID or Binary Columns
--======================================================================================================
SELECT OBJECT_NAME(AC.[object_id]) AS table_name
	, I.name AS index_name
	, I.is_primary_key
	, I.fill_factor
	, AC.name AS column_name
	, T.name AS system_data_type
	, IC.index_column_id AS index_column_order
	, 
	CASE T.name
		WHEN 'uniqueidentifier' THEN 'Clustered index GUID or binary column' 
		WHEN 'binary' THEN 'Clustered index GUID or binary column'
	ELSE 'Additional column in clustered index'
	END AS DBA_notes
FROM sys.index_columns AS IC 
	INNER JOIN sys.indexes AS I 
		ON IC.[object_id] = I.[object_id] AND IC.index_id = I.index_id
	INNER JOIN sys.all_columns AS AC ON IC.[object_id] = AC.[object_id] AND IC.column_id = AC.column_id
	INNER JOIN sys.types AS T ON AC.system_type_id = T.system_type_id AND AC.user_type_id = T.user_type_id 
	INNER JOIN 
		(
			SELECT IC.[object_id]
			FROM sys.index_columns AS IC 
				INNER JOIN sys.all_columns AS AC ON IC.[object_id] = AC.[object_id] AND IC.column_id = AC.column_id
				INNER JOIN sys.types AS T ON AC.system_type_id = T.system_type_id AND AC.user_type_id = T.user_type_id 
			WHERE IC.index_id = 1
				AND T.name IN ('uniqueidentifier', 'binary') 
			GROUP BY IC.[object_id]
		) AS GUID_HITS ON I.[object_id] = GUID_HITS.[object_id] AND I.index_id = 1
ORDER BY 1, IC.index_column_id
OPTION(RECOMPILE);



--======================================================================================
--Listing of All Non-Clustered Indexes w/ GUIDs and Binary Typed Columns in Clustered Index
--======================================================================================
SELECT object_name(AC.[object_id]) AS [table_name]
	, I.name
	, I.index_id
	, I.is_primary_key
	, I.fill_factor
	, AC.name AS column_name
	, T.name AS [system_data_type]
	, IC.index_column_id AS index_column_order 
	, 'Non-Clustered Index w/GUID or Binary' AS notes
FROM sys.index_columns IC 
	INNER JOIN sys.indexes I 
		ON IC.[object_id] = I.[object_id] AND IC.index_id = I.index_id
	INNER JOIN sys.all_columns AC ON IC.[object_id] = AC.[object_id] AND IC.column_id = AC.column_id
	INNER JOIN sys.types T ON AC.system_type_id = T.system_type_id
WHERE I.index_id > 1
	AND T.name IN ('uniqueidentifier', 'binary')
ORDER BY 1;



--======================================================================================================
--Listing of All Non-Clustered Indexes w/ PK Constraint w/ GUIDs and Binary Typed Columns in Clustered Index
--======================================================================================================
SELECT object_name(AC.[object_id]) AS [table_name]
	, I.name
	, I.index_id
	, I.is_primary_key
	, I.fill_factor
	, AC.name AS column_name
	, T.name AS [system_data_type]
	, IC.index_column_id AS index_column_order 
	, 'Non-Clustered Index as Primary Key w/GUID or Binary' AS notes
FROM sys.index_columns IC 
	INNER JOIN sys.indexes I 
		ON IC.[object_id] = I.[object_id] AND IC.index_id = I.index_id
	INNER JOIN sys.all_columns AC ON IC.[object_id] = AC.[object_id] AND IC.column_id = AC.column_id
	INNER JOIN sys.types T ON AC.system_type_id = T.system_type_id
WHERE I.index_id > 1
	AND T.name IN ('uniqueidentifier', 'binary')
	AND I.is_primary_key = 1
ORDER BY 1;




--======================================================================================================
-- Listing of All Columns in Non-Clustered Indexes w/PK Constraint Having GUID or Binary Columns
--======================================================================================================
SELECT OBJECT_NAME(AC.[object_id]) AS table_name
	, I.name AS index_name
	, I.is_primary_key
	, I.fill_factor
	, AC.name AS column_name
	, T.name AS system_data_type
	, IC.index_column_id AS index_column_order
	, 
	CASE T.name
		WHEN 'uniqueidentifier' THEN 'Primary Key Non-Clustered index GUID or binary column' 
		WHEN 'binary' THEN 'Primary Key Non-Clustered index GUID or binary column'
	ELSE 'Additional column in clustered index'
	END AS DBA_notes
FROM sys.index_columns AS IC 
	INNER JOIN sys.indexes AS I 
		ON IC.[object_id] = I.[object_id] AND IC.index_id = I.index_id
	INNER JOIN sys.all_columns AS AC ON IC.[object_id] = AC.[object_id] AND IC.column_id = AC.column_id
	INNER JOIN sys.types AS T ON AC.system_type_id = T.system_type_id AND AC.user_type_id = T.user_type_id 
	INNER JOIN 
		(
			SELECT IC.[object_id], IC.index_id
			FROM sys.index_columns AS IC 
				INNER JOIN sys.all_columns AS AC ON IC.[object_id] = AC.[object_id] AND IC.column_id = AC.column_id
				INNER JOIN sys.types AS T ON AC.system_type_id = T.system_type_id AND AC.user_type_id = T.user_type_id 
				INNER JOIN sys.indexes I ON IC.[object_id] = I.[object_id] AND IC.index_id = I.index_id
			WHERE IC.index_id > 1
				AND T.name IN ('uniqueidentifier', 'binary') 
				AND I.is_primary_key = 1
			GROUP BY IC.[object_id], IC.index_id
		) AS GUID_HITS ON I.[object_id] = GUID_HITS.[object_id] AND I.index_id = GUID_HITS.index_id
WHERE I.index_id > 1
	AND I.is_primary_key = 1
ORDER BY 1, IC.index_column_id
OPTION(RECOMPILE);



SELECT OBJECT_SCHEMA_NAME(SC.object_id) AS schema__name
	, OBJECT_NAME(SC.object_id) AS table__name
	, AC.name AS column__name 
	, SC.type
    , SC.type_desc
    , SC.create_date
    , SC.modify_date
    , SC.is_ms_shipped
    , SC.parent_column_id
    , SC.is_system_named 
FROM sys.default_constraints SC
	LEFT JOIN sys.all_columns AC ON SC.parent_object_id = AC.object_id AND SC.parent_column_id = AC.column_id
WHERE SC.definition = '(newsequentialid())';



SELECT AC.*
FROM sys.all_columns AC
WHERE OBJECT_ID = 565577053


SELECT AC.*
FROM sys.all_columns AC
WHERE OBJECT_NAME(OBJECT_ID) = 