WITH grouped_reads AS
	(
		SELECT ixU.[object_id]
			, SUM(ixU.user_seeks + ixU.user_scans + ixU.user_lookups) AS total_user_reads
			, SUM(ixU.user_updates) AS total_user_writes
		FROM sys.dm_db_index_usage_stats AS ixU 
		WHERE ixU.database_id = db_id() 
			AND ixU.index_id > 0
			AND ixU.user_updates > 0	
		GROUP BY ixU.[object_id]
		HAVING SUM(ixU.user_seeks + ixU.user_scans + ixU.user_lookups) > 0
			AND SUM(ixU.user_updates) > 0
	)

SELECT OBJECT_NAME(ixU.[object_id]) AS object__name
	, sI.indid AS index_id
	, sI.name AS index_name
	, sI.rowcnt AS row_count
	, (ixU.user_seeks + ixU.user_scans + ixU.user_lookups) AS user_reads
	, GR.total_user_reads AS total_reads_for_all_indexes
	, CAST(100 * ((ixU.user_seeks + ixU.user_scans + ixU.user_lookups) / (GR.total_user_reads * 1.0)) AS DECIMAL(4,1)) AS pct_total_reads
	, ixU.user_updates
	, GR.total_user_writes AS total_writes_for_all_indexes
	, CAST((100 * ixU.user_updates / (GR.total_user_writes) * 1.0) AS DECIMAL(4,1)) AS pct_total_updates
FROM sys.dm_db_index_usage_stats AS ixU 
	INNER JOIN sys.sysindexes AS sI ON ixU.[object_id] = sI.id AND ixU.index_id = sI.indid 
	INNER JOIN grouped_reads AS GR ON GR.[object_id] = ixU.[object_id] 
WHERE ixU.database_id = db_id() 
	AND ixU.index_id > 0
	AND ixU.user_updates > 0
	AND sI.rowcnt > 500
ORDER BY 1, 2;
GO


SELECT OBJECT_NAME(I.[object_id]) AS table_name
	, I.index_id
	, I.name AS index_name
	, I.is_primary_key
	, C.name AS column_name
	, T.name AS column_data_type
FROM sys.indexes I 
	INNER JOIN sys.index_columns IC 
		ON IC.index_id = I.index_id 
			AND IC.[object_id] = I.[object_id]
	INNER JOIN sys.columns C ON C.[object_id] = IC.[object_id] 
		AND C.column_id = IC.column_id
	INNER JOIN sys.types T ON C.system_type_id = T.system_type_id
WHERE OBJECT_NAME(I.[object_id]) = 'CNS'
ORDER BY 1,2;


SELECT OBJECT_NAME(I.[object_id]) AS table_name
	, I.index_id
	, I.name AS index_name
	, I.is_primary_key
	, C.name AS column_name
	, T.name AS column_data_type
FROM sys.indexes I 
	INNER JOIN sys.index_columns IC 
		ON IC.index_id = I.index_id 
			AND IC.[object_id] = I.[object_id]
	INNER JOIN sys.columns C ON C.[object_id] = IC.[object_id] 
		AND C.column_id = IC.column_id
	INNER JOIN sys.types T ON C.system_type_id = T.system_type_id
WHERE OBJECT_NAME(I.[object_id]) = 'TransactionDetailsRUAvail'
ORDER BY 1,2;