--=====================================================
--ORDER BY EXECUTION PERIOD AND ORDER OF EXECUTION
--=====================================================

SELECT DatabaseName
	, SchemaName
       , ObjectName
       , ObjectType
       , IndexName
       , CASE IndexType
			WHEN 1 THEN 'Clustered'
			WHEN 2 THEN 'Nonclustered'
			ELSE 'Other'
		END AS IndexType
       , CASE 
			WHEN Command LIKE '%REORGANIZE%' THEN 'REORGANIZE'
			ELSE 'REBUILD'
		END AS Command
       , CommandType
       , StartTime
       , EndTime
	   , CASE 
			WHEN DATEPART(hh,StartTime) < 12 THEN DateAdd(Day, Datediff(Day,0, StartTime -1), 0)
			ELSE DateAdd(Day, Datediff(Day,0, StartTime -0), 0) 
		END AS execution_date
FROM iDBA.dbo.CommandLog
WHERE CommandType = 'ALTER_INDEX'
ORDER BY 1, 11 DESC, ID DESC;
GO


--=====================================================
--GROUP BY INDEX AND ACTION
--=====================================================
DECLARE @history_days DECIMAL(4,1)
SELECT @history_days = DATEDIFF(dd, MIN(startTime), MAX(startTime))
FROM iDBA.dbo.CommandLog 
WHERE CommandType = 'ALTER_INDEX';

SELECT @history_days AS number_of_days_history

	CREATE TABLE #TableData (
		[I1] SMALLINT NOT NULL,
		[SchemaName] NVARCHAR(128) NOT NULL,
		[TableName]  NVARCHAR (128) NOT NULL,
		[RowCount] BIGINT NOT NULL,
		[Reserved] BIGINT NOT NULL,
		[Data] BIGINT NOT NULL,
		[IndexSize] BIGINT NOT NULL,
		[Unused] BIGINT NOT NULL);
		
	INSERT INTO #TableData (I1, [SchemaName], [TableName],[RowCount],[Reserved],[Data], [IndexSize],[Unused])
	SELECT 	(row_number() over(order by (A.reserved + ISNULL(B.reserved,0)) desc))%2 as l1,
				A.[SchemaName], 
				Obj.name AS [Tablename],
				A.rows as [RowCount],
				(A.reserved + ISNULL(B.reserved,0))* 8 AS [Reserved(KB)],
				A.data * 8 AS [Data(KB)],
				(CASE 
					WHEN (A.used + ISNULL(B.used,0)) > A.data 
						THEN (A.used + ISNULL(B.used,0)) - A.data 
						ELSE 0 END) * 8 AS [IndexSize(KB)],
				(CASE 
					WHEN (A.reserved + ISNULL(B.reserved,0)) > A.used 
						THEN (A.reserved + ISNULL(B.reserved,0)) - A.used 
						ELSE 0 END) * 8 AS [Unused(KB)]
	FROM (
		SELECT OBJECT_SCHEMA_NAME(PartStat.object_id) AS [SchemaName]
			, PartStat.object_id,
				SUM (CASE
						WHEN (PartStat.index_id < 2) THEN row_count
						ELSE 0
						END) AS [rows],
				SUM (PartStat.reserved_page_count) AS reserved,
				SUM (CASE
					WHEN (PartStat.index_id < 2) 
						THEN (PartStat.in_row_data_page_count + PartStat.lob_used_page_count + PartStat.row_overflow_used_page_count)
					ELSE (PartStat.lob_used_page_count + PartStat.row_overflow_used_page_count)
					END) AS data,
				SUM (PartStat.used_page_count) AS used
		FROM sys.dm_db_partition_stats PartStat
					GROUP BY PartStat.object_id) AS A
		LEFT OUTER JOIN
			(SELECT Tab.parent_id,
				SUM(PartStat.reserved_page_count) AS reserved,
				SUM(PartStat.used_page_count) AS used
			FROM sys.dm_db_partition_stats PartStat
				INNER JOIN sys.internal_tables Tab ON (Tab.object_id = PartStat.object_id)
			WHERE Tab.internal_type IN (202,204)
				GROUP BY Tab.parent_id) AS B ON (B.parent_id = A.object_id)
			INNER JOIN sys.all_objects Obj  ON ( A.object_id = Obj.object_id )
				WHERE Obj.type <> N'S' 
					AND Obj.type <> N'IT' 
					AND Obj.Name NOT LIKE 'tmp%'
					AND A.rows > 0
				ORDER BY Obj.name;
				


WITH History AS
	(
	
		SELECT SchemaName
		   , ObjectName
		   , ObjectType
		   , IndexName
		   , CASE IndexType
				WHEN 1 THEN 'Clustered'
				WHEN 2 THEN 'Nonclustered'
				ELSE 'Other'
			END AS IndexType
		   , CASE 
				WHEN Command LIKE '%REORGANIZE%' THEN 'REORGANIZE'
				ELSE 'REBUILD'
			END AS Command
		   , CommandType
		   , StartTime
		   , EndTime
		   , CASE 
				WHEN DATEPART(hh,StartTime) < 12 THEN DateAdd(Day, Datediff(Day,0, StartTime -1), 0)
				ELSE DateAdd(Day, Datediff(Day,0, StartTime -0), 0) 
			END AS execution_date
	FROM iDBA.dbo.CommandLog
	WHERE CommandType = 'ALTER_INDEX'
)



SELECT History.SchemaName
	, History.ObjectName
	, History.ObjectType
	, History.IndexName
	, History.IndexType 
	, #TableData.[RowCount] AS table_rows
	--, #TableData.Reserved/1024/1024 AS Reserved_Gb
	--, #TableData.Data/1024/1024 AS Data_Gb
	, #TableData.IndexSize/1024/1024 AS index_gb
	, History.Command       
	, COUNT(History.SchemaName) AS execution_count
	, COUNT(History.SchemaName)/@history_days*100 AS action_ratio
FROM History 
	LEFT JOIN #TableData ON History.SchemaName = #TableData.SchemaName	
		AND History.ObjectName = #TableData.TableName
	--LEFT JOIN #TableData ON History.ObjectName = #TableData.TableName
GROUP BY History.SchemaName
	, History.ObjectName
	, History.ObjectType
	, History.IndexName
	, History.IndexType 
	, #TableData.[RowCount] 
	, #TableData.IndexSize     
	, History.Command 
ORDER BY 10 DESC
	, History.SchemaName
	, History.ObjectName
	, History.ObjectType
	, History.IndexType
	, History.IndexName      
	, History.Command;

DROP TABLE #TableData;
GO


--=====================================================
--GROUP BY INDEX ONLY
--=====================================================
DECLARE @history_days DECIMAL(4,1)
SELECT @history_days = DATEDIFF(dd, MIN(startTime), MAX(startTime))
				FROM iDBA.dbo.CommandLog 
				WHERE CommandType = 'ALTER_INDEX';

	CREATE TABLE #TableData (
		[I1] SMALLINT NOT NULL,
		[SchemaName] NVARCHAR(128) NOT NULL,
		[TableName]  NVARCHAR (128) NOT NULL,
		[RowCount] BIGINT NOT NULL,
		[Reserved] BIGINT NOT NULL,
		[Data] BIGINT NOT NULL,
		[IndexSize] BIGINT NOT NULL,
		[Unused] BIGINT NOT NULL);
		
	INSERT INTO #TableData (I1, [SchemaName], [TableName],[RowCount],[Reserved],[Data], [IndexSize],[Unused])
	SELECT 	(row_number() over(order by (A.reserved + ISNULL(B.reserved,0)) desc))%2 as l1,
				A.[SchemaName], 
				Obj.name AS [Tablename],
				A.rows as [RowCount],
				(A.reserved + ISNULL(B.reserved,0))* 8 AS [Reserved(KB)],
				A.data * 8 AS [Data(KB)],
				(CASE 
					WHEN (A.used + ISNULL(B.used,0)) > A.data 
						THEN (A.used + ISNULL(B.used,0)) - A.data 
						ELSE 0 END) * 8 AS [IndexSize(KB)],
				(CASE 
					WHEN (A.reserved + ISNULL(B.reserved,0)) > A.used 
						THEN (A.reserved + ISNULL(B.reserved,0)) - A.used 
						ELSE 0 END) * 8 AS [Unused(KB)]
	FROM (
		SELECT OBJECT_SCHEMA_NAME(PartStat.object_id) AS [SchemaName]
			, PartStat.object_id,
				SUM (CASE
						WHEN (PartStat.index_id < 2) THEN row_count
						ELSE 0
						END) AS [rows],
				SUM (PartStat.reserved_page_count) AS reserved,
				SUM (CASE
					WHEN (PartStat.index_id < 2) 
						THEN (PartStat.in_row_data_page_count + PartStat.lob_used_page_count + PartStat.row_overflow_used_page_count)
					ELSE (PartStat.lob_used_page_count + PartStat.row_overflow_used_page_count)
					END) AS data,
				SUM (PartStat.used_page_count) AS used
		FROM sys.dm_db_partition_stats PartStat
					GROUP BY PartStat.object_id) AS A
		LEFT OUTER JOIN
			(SELECT Tab.parent_id,
				SUM(PartStat.reserved_page_count) AS reserved,
				SUM(PartStat.used_page_count) AS used
			FROM sys.dm_db_partition_stats PartStat
				INNER JOIN sys.internal_tables Tab ON (Tab.object_id = PartStat.object_id)
			WHERE Tab.internal_type IN (202,204)
				GROUP BY Tab.parent_id) AS B ON (B.parent_id = A.object_id)
			INNER JOIN sys.all_objects Obj  ON ( A.object_id = Obj.object_id )
				WHERE Obj.type <> N'S' 
					AND Obj.type <> N'IT' 
					AND Obj.Name NOT LIKE 'tmp%'
					AND A.rows > 0
				ORDER BY Obj.name;
				
--SELECT * FROM #TableData ORDER BY SchemaName, TableName;

WITH History AS
	(
	
		SELECT SchemaName
		   , ObjectName
		   , ObjectType
		   , IndexName
		   , CASE IndexType
				WHEN 1 THEN 'Clustered'
				WHEN 2 THEN 'Nonclustered'
				ELSE 'Other'
			END AS IndexType
		 --  , CASE 
			--	WHEN Command LIKE '%REORGANIZE%' THEN 'REORGANIZE'
			--	ELSE 'REBUILD'
			--END AS Command
		   , CommandType
		   , StartTime
		   , EndTime
		   , CASE 
				WHEN DATEPART(hh,StartTime) < 12 THEN DateAdd(Day, Datediff(Day,0, StartTime -1), 0)
				ELSE DateAdd(Day, Datediff(Day,0, StartTime -0), 0) 
			END AS execution_date
	FROM iDBA.dbo.CommandLog
	WHERE CommandType = 'ALTER_INDEX'
	)



SELECT History.SchemaName
	, History.ObjectName
	, History.ObjectType
	, History.IndexName
	, History.IndexType 
	, #TableData.[RowCount] AS table_rows
	--, #TableData.Reserved/1024/1024 AS Reserved_Gb
	--, #TableData.Data/1024/1024 AS Data_Gb
	, #TableData.IndexSize/1024/1024 AS index_gb
	--, History.Command       
	, COUNT(History.SchemaName) AS execution_count
	, COUNT(History.SchemaName)/@history_days*100 AS action_ratio
FROM History 
	LEFT JOIN #TableData ON History.SchemaName = #TableData.SchemaName	
		AND History.ObjectName = #TableData.TableName
	--LEFT JOIN #TableData ON History.ObjectName = #TableData.TableName
GROUP BY History.SchemaName
	, History.ObjectName
	, History.ObjectType
	, History.IndexName
	, History.IndexType 
	, #TableData.[RowCount] 
	, #TableData.IndexSize     
	--, History.Command 
ORDER BY 9 DESC
	, History.SchemaName
	, History.ObjectName
	, History.ObjectType
	, History.IndexType
	, History.IndexName;      
	--, History.Command;


--SELECT [SchemaName], [TableName],[RowCount],[Reserved] AS [Reserved(KB)],[Data] AS [Data(KB)],[IndexSize] AS [IndexSize(KB)],[Unused] AS [Unused(KB)] 
--	FROM #TableData
--ORDER BY [Reserved] DESC; --[TableName]

DROP TABLE #TableData;
GO