--========================
-- QUESTIONS TO ANSWER
--========================

--Q1================================================================
-- UNUSED INDEXES
--==================================================================
--sys.dm_db_index_usage_stats where SUM(reads) = 0 

SELECT 'Q1: Unused Indexes:' AS question_addressed 
	, OBJECT_NAME(ixU.[object_id]) AS object__name
	, sI.indid AS index_id
	, sI.name AS index_name
	, sI.rowcnt AS row_count
	, ixU.user_updates
FROM sys.dm_db_index_usage_stats AS ixU 
	INNER JOIN sys.sysindexes AS sI ON ixU.[object_id] = sI.id AND ixU.index_id = sI.indid 
WHERE ixU.database_id = db_id() 
	AND ixU.index_id > 0
	AND (ixU.user_seeks + ixU.user_scans + ixU.user_lookups) = 0 
	AND ixU.user_updates > 0 
ORDER BY ixU.user_updates DESC, sI.rowcnt DESC;
GO








--Q2================================================================
-- POSSIBLY UNNECESSARY INDEXES
--==================================================================
--sys.dm_db_index_usage_stats where SUM(reads) very low compared to SUM(writes)

SELECT 'Q2: Possible Unncessary Indexes:' AS question_addressed 
	, OBJECT_NAME(ixU.[object_id]) AS object__name
	, sI.indid AS index_id
	, sI.name AS index_name
	, sI.rowcnt AS row_count
	, (ixU.user_seeks + ixU.user_scans + ixU.user_lookups) AS [user_reads]
	, ixU.user_updates
	, (1.0) * (ixU.user_seeks + ixU.user_scans + ixU.user_lookups) / ixU.user_updates AS [reads_per_write]
FROM sys.dm_db_index_usage_stats AS ixU 
	INNER JOIN sys.sysindexes AS sI ON ixU.[object_id] = sI.id AND ixU.index_id = sI.indid 
WHERE ixU.database_id = db_id() 
	AND ixU.index_id > 0
	AND ixU.user_updates > 0
	AND sI.rowcnt > 0
ORDER BY 8 ASC, ixU.user_updates DESC, sI.rowcnt DESC, 2, 3;
GO









--Q3================================================================
-- CLUSTERING KEY CHOICES 3.0 - Bad Key
--==================================================================
--sys.dm_db_index_usage_stats where clustering key SUM(reads) is lower than other non-clustering keys
--?  does having the clustered index on the most-read column(s) benefit over the lowest cardinality column?--

WITH grouped_reads AS
	(
		SELECT ixU.[object_id]
			, SUM(ixU.user_seeks + ixU.user_scans + ixU.user_lookups) AS total_user_reads
			, SUM(ixU.user_updates) AS total_user_writes
		FROM sys.dm_db_index_usage_stats AS ixU 
		WHERE ixU.database_id = db_id() 
			AND ixU.index_id > 0
			AND ixU.user_updates > 0	
		GROUP BY ixU.[object_id]
		HAVING SUM(ixU.user_seeks + ixU.user_scans + ixU.user_lookups) > 0
			AND SUM(ixU.user_updates) > 0
	)

SELECT 'Q3: Look at Possible Clustering Key Changes:' 
	, OBJECT_NAME(ixU.[object_id]) AS object__name
	, sI.indid AS index_id
	, sI.name AS index_name
	, sI.rowcnt AS row_count
	, (ixU.user_seeks + ixU.user_scans + ixU.user_lookups) AS user_reads
	, GR.total_user_reads AS total_reads_for_all_indexes
	, CAST(100 * ((ixU.user_seeks + ixU.user_scans + ixU.user_lookups) / (GR.total_user_reads * 1.0)) AS DECIMAL(4,1)) AS pct_total_reads
	, ixU.user_updates
	, GR.total_user_writes AS total_writes_for_all_indexes
	, CAST((100 * ixU.user_updates / (GR.total_user_writes) * 1.0) AS DECIMAL(4,1)) AS pct_total_updates
FROM sys.dm_db_index_usage_stats AS ixU 
	INNER JOIN sys.sysindexes AS sI ON ixU.[object_id] = sI.id AND ixU.index_id = sI.indid 
	INNER JOIN grouped_reads AS GR ON GR.[object_id] = ixU.[object_id] 
WHERE ixU.database_id = db_id() 
	AND ixU.index_id > 0
	AND ixU.user_updates > 0
	AND sI.rowcnt > 2000
ORDER BY 2, 3;
GO


SELECT OBJECT_NAME(I.[object_id]) AS table_name
	, I.index_id
	, I.name AS index_name
	, I.is_primary_key
	, C.name AS column_name
	, T.name AS column_data_type
FROM sys.indexes I 
	INNER JOIN sys.index_columns IC 
		ON IC.index_id = I.index_id 
			AND IC.[object_id] = I.[object_id]
	INNER JOIN sys.columns C ON C.[object_id] = IC.[object_id] 
		AND C.column_id = IC.column_id
	INNER JOIN sys.types T ON C.system_type_id = T.system_type_id
WHERE OBJECT_NAME(I.[object_id]) = 'CNS'
ORDER BY 1,2;


SELECT OBJECT_NAME(I.[object_id]) AS table_name
	, I.index_id
	, I.name AS index_name
	, I.is_primary_key
	, C.name AS column_name
	, T.name AS column_data_type
FROM sys.indexes I 
	INNER JOIN sys.index_columns IC 
		ON IC.index_id = I.index_id 
			AND IC.[object_id] = I.[object_id]
	INNER JOIN sys.columns C ON C.[object_id] = IC.[object_id] 
		AND C.column_id = IC.column_id
	INNER JOIN sys.types T ON C.system_type_id = T.system_type_id
WHERE OBJECT_NAME(I.[object_id]) = 'TransactionDetailsRUAvail'
ORDER BY 2,3;





--Q3.1==============================================================
-- CLUSTERING KEY CHOICES 3.1 - No Clustered Index
--==================================================================
--sys.dm_db_index_usage_stats where SUM(reads) for index_id = 0 is HIGH

SELECT 'Q2.5: High Activity Heaps:'
	, SCHEMA_NAME(O.schema_id) AS [schema_name]
	, ixU.[object_id]
	, OBJECT_NAME(ixU.[object_id] ) AS table_name
	, P.partition_number
    , P.rows AS table_rows
    , (ixU.user_seeks + ixU.user_scans + ixU.user_lookups) AS total_user_reads
	, ixU.user_updates AS total_user_writes
    , ixU.last_user_seek
    , ixU.last_user_scan
    , ixU.last_user_lookup
FROM sys.indexes I
	INNER JOIN sys.objects O 
		ON I.[object_id] = O.[object_id]
    INNER JOIN sys.partitions p 
		ON I.[object_id] = P.[object_id] 
			AND I.index_id = P.index_id
		INNER JOIN sys.dm_db_index_usage_stats ixU 
			ON I.[object_id] = ixU.[object_id] 
				AND I.index_id = ixU.index_id
WHERE I.type_desc = 'HEAP'
	AND (ixU.user_seeks + ixU.user_scans + ixU.user_lookups) > 0
		AND ixU.user_updates > 0
	AND ixU.database_id = db_id()
ORDER BY (ixU.user_seeks + ixU.user_scans + ixU.user_lookups) DESC 
	, ixU.user_updates ASC

	






--Q4================================================================
-- INCORRECT FILL FACTOR
--==================================================================
--sys.dm_db_index_operational_stats order by leaf_allocation_count DESC
-- corollate to average_fragmentation_in_pct from sys.dm_db_index_physical_stats
-- remember leaf_allocation_count is reset after a rebuild so also looking at frequency of rebuilds from command_log is important
--?  ?--

SELECT DATEDIFF(dd, MIN(startTime), MAX(startTime)) AS history_days_in_commandlog
FROM iDBA.dbo.CommandLog 
WHERE CommandType = 'ALTER_INDEX'
	AND DatabaseName = 'ClientVantage';

CREATE TABLE ##Index_History
	(
		the_database sysname,
		the_schema sysname,
		the_object sysname,
		the_index sysname,
		index_type VARCHAR(13),
		fill_factor tinyint,
		rebuild_count int NULL,
		reorg_count int NULL
	);

CREATE TABLE ##Indexes
	(
		the_schema sysname,
		the_object sysname,
		the_index sysname,
		fill_factor tinyint
	);

INSERT INTO ##Indexes
	(
	the_schema
	, the_object
	, the_index
	, fill_factor
	)

SELECT OBJECT_SCHEMA_NAME(SO.object_id, db_id('ClientVantage')) AS the_schema
	, SO.name AS the_object
	, SI.name AS the_index
	, SI.fill_factor
FROM ClientVantage.sys.objects SO
	INNER JOIN ClientVantage.sys.indexes SI
		ON SO.object_id = SI.object_id
WHERE SI.index_id > 0;

INSERT INTO ##Index_History
         (the_database
        , the_schema
        , the_object
        , the_index
        , index_type
        , fill_factor
        , rebuild_count
        , reorg_count
         )

SELECT C.DatabaseName
    , C.SchemaName
    , C.ObjectName
    , C.IndexName
    , CASE C.IndexType
		WHEN 1 THEN 'Clustered'
		ELSE 'Non-Clustered'
	END AS IndexType
	, IX.fill_factor
	, COUNT(C.ID) AS rebuild_count
	, 0 AS reorg_count
FROM iDBA.dbo.CommandLog C 
	LEFT JOIN ##Indexes IX ON C.SchemaName = IX.the_schema
		AND C.ObjectName = IX.the_object
		AND C.IndexName = IX.the_index
WHERE C.CommandType = 'ALTER_INDEX'
	AND C.ObjectType = 'U'
	AND C.Command LIKE '%REBUILD%'
	AND C.DatabaseName = 'ClientVantage'
GROUP BY C.DatabaseName
    , C.SchemaName
    , C.ObjectName
    , C.IndexName
    , IndexType
	, IX.fill_factor;


--============================================================================
-- LAST INDEX REBUILD FOR A SPECIFIC INDEX
--============================================================================
WITH rebuild_history AS
	(
		SELECT ROW_NUMBER() OVER( PARTITION BY CL.ObjectName, CL.IndexName ORDER BY (CL.EndTime) DESC) AS rn
			, CL.ObjectName
			, CL.indexName
			, CL.EndTime
		FROM iDBA.[dbo].[CommandLog] AS CL 
		WHERE CL.DatabaseName = 'ClientVantage' 
			AND CL.Command LIKE '%REBUILD%' 
			AND CL.ObjectType = 'U'
	)

SELECT rn
    , ObjectName
    , IndexName
    , EndTime AS last_rebuild_date 
FROM rebuild_history 
WHERE rn = 1 
ORDER BY ObjectName, IndexName	


--============================================================================
-- INDEX REORGANIZE (ONLY) COUNT LAST 31 DAYS
--============================================================================
INSERT INTO ##Index_History
         (the_database
        , the_schema
        , the_object
        , the_index
        , index_type
        , fill_factor
        , rebuild_count
        , reorg_count
         )

SELECT C.DatabaseName
    , C.SchemaName
    , C.ObjectName
    , C.IndexName
    , CASE C.IndexType
		WHEN 1 THEN 'Clustered'
		ELSE 'Non-Clustered'
	END AS IndexType
	, IX.fill_factor
	, 0 AS rebuild_count
	, COUNT(C.ID) AS reorg__count
FROM iDBA.dbo.CommandLog C 
	LEFT JOIN ##Indexes IX ON C.SchemaName = IX.the_schema
		AND C.ObjectName = IX.the_object
		AND C.IndexName = IX.the_index
	LEFT JOIN ##Index_History IH
		ON C.DatabaseName = IH.the_database
			AND C.SchemaName = IH.the_schema
			AND C.ObjectName = IH.the_object
			AND C.IndexName = IH.the_index
WHERE C.CommandType = 'ALTER_INDEX'
	AND C.ObjectType = 'U'
	AND C.Command LIKE '%REORGANIZE%'
	AND C.DatabaseName = 'ClientVantage'
	AND IH.the_database IS NULL
GROUP BY C.DatabaseName
    , C.SchemaName
    , C.ObjectName
    , C.IndexName
    , IndexType
	, IX.fill_factor;

--========================================================
-- ACCOUNT FOR INDEXES BOTH REBUILT AND REORGANIZED
--========================================================
UPDATE ##Index_History
SET reorg_count = C2.reorganize_count
FROM ##Index_History IH
	INNER JOIN
		(
			SELECT C.DatabaseName
				, C.SchemaName
				, C.ObjectName
				, C.IndexName
				, COUNT(C.ID) AS reorganize_count
				 FROM iDBA.dbo.CommandLog C	
			WHERE C.CommandType = 'ALTER_INDEX'
				AND C.ObjectType = 'U'
				AND C.Command LIKE '%REORGANIZE%'
				AND C.DatabaseName = 'ClientVantage'
			GROUP BY C.DatabaseName
				, C.SchemaName
				, C.ObjectName
				, C.IndexName
		) C2 ON IH.the_database = C2.DatabaseName
			AND IH.the_schema = C2.SchemaName 
			AND IH.the_object = C2.ObjectName   
			AND IH.the_index = C2.IndexName 
WHERE IH.rebuild_count > 0


--============================================================================
-- RETURN THE RESULTS
--============================================================================
SELECT 'Q4: Poor Fill Factor Settings:' 
	, the_database
    , the_schema
    , the_object
    , the_index
    , index_type
    , fill_factor
    , rebuild_count
    , reorg_count 
FROM ##Index_History 
ORDER BY the_database
	, rebuild_count DESC
	, reorg_count DESC
	, the_object
	, the_index

--============================================================================
-- CLEANUP THE MESS
--============================================================================
IF EXISTS (SELECT name FROM tempdb.sys.objects WHERE name = '##Index_History')
BEGIN
	DROP TABLE ##Index_History
END

IF EXISTS (SELECT name FROM tempdb.sys.objects WHERE name = '##Indexes')
BEGIN
	DROP TABLE ##Indexes
END








--Q5================================================================
-- GUID DATA TYPES IN INDEXES AFFECTING FRAGMENTATION & REBUILDS
--==================================================================
--Look at GUID Script Results
--Then follow same process as check for INCORRECT FILL FACTOR to see if fragmentation & page splits are occuring due to GUIDs

-- corollate to average_fragmentation_in_pct from sys.dm_db_index_physical_stats
-- remember leaf_allocation_count is reset after a rebuild so also looking at frequency of rebuilds from command_log is important


CREATE TABLE #guid_clustered_indexes (table_name sysname, index_name sysname, is_primary_key BIT, fill_factor TINYINT, column_name sysname, system_data_type sysname, index_column_order INT, notes VARCHAR(100))
INSERT INTO #guid_clustered_indexes
      (
       table_name
     , index_name
     , is_primary_key
     , fill_factor
     , column_name
     , system_data_type
     , index_column_order
     , notes
      )
SELECT object_name(AC.[object_id]) AS [table_name]
	, I.name AS index_name
	, I.is_primary_key
	, I.fill_factor
	, AC.name AS column_name
	, T.name AS [system_data_type]
	, IC.index_column_id AS index_column_order 
	, 'Clustered Index w/GUID' AS notes
FROM sys.index_columns IC 
	INNER JOIN sys.indexes I 
		ON IC.[object_id] = I.[object_id] AND IC.index_id = I.index_id
	INNER JOIN sys.all_columns AC ON IC.[object_id] = AC.[object_id] AND IC.column_id = AC.column_id
	INNER JOIN sys.types T ON AC.system_type_id = T.system_type_id
WHERE I.index_id = 1
	AND T.name = 'uniqueidentifier';

CREATE TABLE #page_splits (table_name sysname, index_id INT, index_name sysname, fill_factor TINYINT, index_type NVARCHAR(60), partition_number INT, leaf_allocation_count BIGINT, nonleaf_allocation_count BIGINT)

INSERT INTO #page_splits
      (
       table_name
     , index_id
     , index_name
     , fill_factor
     , index_type
     , partition_number
     , leaf_allocation_count
     , nonleaf_allocation_count
      )

SELECT SO.[name] AS table_name
	, I.index_id
	, I.[name] AS index_name
	, I.fill_factor
	, I.type_desc AS index_type
	, ixO.partition_number
	, ixO.leaf_allocation_count
	, ixO.nonleaf_allocation_count
FROM sys.dm_db_index_operational_stats (db_id(), NULL, NULL, NULL) AS ixO
	INNER JOIN sys.indexes I 
		ON ixO.[object_id] = I.[object_id] 
			AND ixO.[index_id] = I.[index_id] 
	INNER JOIN sys.objects SO 
		ON I.[object_id] = SO.[object_id]
WHERE I.index_id > 0
	AND SO.is_ms_shipped = 0
ORDER BY ixO.leaf_allocation_count DESC;


CREATE TABLE #frag_status (table_name sysname, index_id INT, index_name sysname, partition_number INT, frag_pct TINYINT, page_count INT)

INSERT INTO #frag_status
      (
       table_name
     , index_id
     , index_name
     , partition_number
     , frag_pct
     , page_count
      )

SELECT O.[name] AS table_name
	, I.index_id
	, I.[name] AS index_name
	, ixP.partition_number
	, CAST(ixP.avg_fragmentation_in_percent AS TINYINT) AS frag_pct
	, ixP.page_count
FROM sys.dm_db_index_physical_stats (db_id(), NULL, NULL, NULL, 'limited') AS ixP
	INNER JOIN sys.indexes AS I 
		ON ixP.[object_id] = I.[object_id] 
			AND ixP.[index_id] = I.[index_id] 
	INNER JOIN sys.objects O 
		ON I.[object_id] = O.[object_id]
WHERE ixP.index_id > 0
	AND ixP.page_count > 500
	AND ixP.avg_fragmentation_in_percent > 15
ORDER BY ixP.avg_fragmentation_in_percent;



CREATE TABLE #rebuild_history (schema_name sysname, table_name sysname, index_name sysname, index_type VARCHAR(20), rebuild_count TINYINT)

INSERT INTO #rebuild_history
      (
       schema_name
     , table_name
     , index_name
     , index_type
     , rebuild_count
      )

SELECT C.SchemaName AS schema_name
    , C.ObjectName AS table_name
    , C.IndexName AS index_name
    , CASE C.IndexType
		WHEN 1 THEN 'Clustered'
		ELSE 'Non-Clustered'
	END AS index_type
	, COUNT(C.ID) AS rebuild_count
FROM iDBA.dbo.CommandLog C 
WHERE C.CommandType = 'ALTER_INDEX'
	AND C.ObjectType = 'U'
	AND C.Command LIKE '%REBUILD%'
	AND C.DatabaseName = 'ClientVantage'
GROUP BY C.DatabaseName
    , C.SchemaName
    , C.ObjectName
    , C.IndexName
    , IndexType;






WITH last_rebuild AS
	(
		SELECT ROW_NUMBER() OVER( PARTITION BY CL.ObjectName, CL.IndexName ORDER BY (CL.EndTime) DESC) AS rn
			, CL.ObjectName AS table_name
			, CL.indexName AS index_name
			, CL.EndTime AS last_rebuild_time
			, CL.ExtendedInfo
		FROM iDBA.[dbo].[CommandLog] AS CL 
		WHERE CL.DatabaseName = 'ClientVantage' 
			AND CL.Command LIKE '%REBUILD%' 
			AND CL.ObjectType = 'U'
	)

SELECT 'Q5: GUIDs in Clustered Index:' 
	, GUIDS.table_name
    , GUIDS.index_name
    , GUIDS.is_primary_key
    , SPLITS.index_type
	, GUIDS.fill_factor
	, FRAG.frag_pct
	, FRAG.page_count
	, FRAG.partition_number
    , GUIDS.column_name
    , GUIDS.system_data_type
    , GUIDS.index_column_order
	, REBUILDS.rebuild_count
	, COALESCE(LASTREBUILD.last_rebuild_time, 'None in last 30 days') AS last_rebuild_time
	, SPLITS.partition_number
	, SPLITS.leaf_allocation_count
	, SPLITS.nonleaf_allocation_count
	, LASTREBUILD.ExtendedInfo
	, GUIDS.notes
FROM #guid_clustered_indexes AS GUIDS
	INNER JOIN #rebuild_history AS REBUILDS ON REBUILDS.table_name = GUIDS.table_name AND REBUILDS.index_name = GUIDS.index_name
	INNER JOIN #page_splits SPLITS ON SPLITS.table_name = GUIDS.table_name AND SPLITS.index_name = GUIDS.index_name
	LEFT JOIN #frag_status FRAG ON FRAG.table_name = GUIDS.table_name AND FRAG.index_name = GUIDS.index_name
	LEFT JOIN last_rebuild LASTREBUILD ON LASTREBUILD.table_name = GUIDS.table_name AND LASTREBUILD.index_name = GUIDS.index_name
WHERE LASTREBUILD.rn = 1 OR LASTREBUILD.rn IS NULL
ORDER BY REBUILDS.rebuild_count DESC, SPLITS.leaf_allocation_count DESC, GUIDS.index_column_order ASC;


DROP TABLE #guid_clustered_indexes;
DROP TABLE #page_splits;
DROP TABLE #frag_status;
DROP TABLE #rebuild_history;


--============================================================